package org.jeecg.modules.adweb.seo.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.xkcoding.http.util.StringUtil;

import jakarta.annotation.Resource;

import lombok.extern.slf4j.Slf4j;

import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellType;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.jeecg.modules.adweb.common.util.ListUtil;
import org.jeecg.modules.adweb.seo.entity.*;
import org.jeecg.modules.adweb.seo.mapper.SeoKeywordsMapper;
import org.jeecg.modules.adweb.seo.service.*;
import org.jeecg.modules.adweb.seo.vo.RankInfoVO;
import org.jeecg.modules.adweb.seo.vo.SeoRankInfoVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.ParseException;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author Zenas
 */
@Slf4j
@Service
public class SearchKeywordsServiceImpl implements ISearchKeywordsService {

    @Resource private SeoKeywordsMapper seoKeywordsMapper;

    @Autowired private ISeoKeywordsService seoKeywordsService;

    @Autowired private ISeoKeywordsSerpService seoKeywordsSerpService;

    /**
     * 获得综合统计的数据
     *
     * @param siteCode 站点code
     * @param subscriptionId 订阅表id
     * @return 综合统计的数据
     */
    @Override
    public ComprehensiveStatistics getComprehensiveInfo(String siteCode, String subscriptionId) {
        ComprehensiveStatistics comprehensiveStatistics = new ComprehensiveStatistics();

        return comprehensiveStatistics;
    }

    @Override
    public Map getRankInfo(String siteCode, String historyId) throws ParseException {
        List<SeoRankInfoVO> seoRankInfos = this.getSeoRankInfo(siteCode, historyId);

        Map map = new HashMap<>();
        map.put("appointKeyword", seoRankInfos.get(0));
        map.put("longTailKeyword", seoRankInfos.get(1));
        return map;
    }

    /**
     * 获取一个站点的关键词排名数量
     *
     * @param siteCode 站点code
     * @return 关键词排名数量
     */
    @Override
    public List<SeoRankInfoVO> getSeoRankInfo(String siteCode, String historyId)
            throws ParseException {
        // 排名信息
        SeoRankInfoVO appointRankInfo = new SeoRankInfoVO();
        SeoRankInfoVO longTailRankInfo = new SeoRankInfoVO();
        // 1-10
        Map<String, Integer> first = dealKeywordsRankData(siteCode, historyId, 1, 10);
        if (first.containsKey("appoint")) {
            appointRankInfo.setFirstNum(first.get("appoint"));
        }
        if (first.containsKey("longTail")) {
            longTailRankInfo.setFirstNum(first.get("longTail"));
        }

        // 11-30
        Map<String, Integer> second = dealKeywordsRankData(siteCode, historyId, 11, 30);
        if (second.containsKey("appoint")) {
            appointRankInfo.setSecondNum(second.get("appoint"));
        }
        if (second.containsKey("longTail")) {
            longTailRankInfo.setSecondNum(second.get("longTail"));
        }
        // 31-100
        Map<String, Integer> third = dealKeywordsRankData(siteCode, historyId, 31, 100);
        if (third.containsKey("appoint")) {
            appointRankInfo.setThirdType(third.get("appoint"));
        }
        if (third.containsKey("longTail")) {
            longTailRankInfo.setThirdType(third.get("longTail"));
        }
        List<SeoRankInfoVO> list = new ArrayList<>();
        list.add(appointRankInfo);
        list.add(longTailRankInfo);
        return list;
    }

    private Map<String, Integer> dealKeywordsRankData(
            String siteCode, String historyId, Integer rankStart, Integer rankEnd) {
        Map<String, Integer> KeywordsRankMap = new HashMap<>();

        // 指定词
        List<String> appointDateList =
                seoKeywordsService.getKeywordsLastSevenDays(siteCode, 1, historyId);
        String appointDatesStr = "";
        if (ListUtil.notEmpty(appointDateList)) {
            for (int i = 0; i < appointDateList.size(); i++) {
                if (i == appointDateList.size() - 1) {
                    appointDatesStr += "'" + appointDateList.get(i) + "'";
                } else {
                    appointDatesStr += "'" + appointDateList.get(i) + "'" + ",";
                }
            }
            appointDatesStr = "(" + appointDatesStr + ")";

            // 指定词
            RankInfoVO appointRankInfo =
                    seoKeywordsMapper.queryKeywordNumByRankRange(
                            siteCode, historyId, rankStart, rankEnd, appointDatesStr, "1");
            if (appointRankInfo != null) {
                KeywordsRankMap.put("appoint", appointRankInfo.getKeywordNum());
            }
        }

        // 长尾词
        List<String> dateList = seoKeywordsService.getKeywordsLastSevenDays(siteCode, 2, historyId);
        String dateStr = "";
        if (ListUtil.notEmpty(dateList)) {
            for (int i = 0; i < dateList.size(); i++) {
                if (i == dateList.size() - 1) {
                    dateStr += "'" + dateList.get(i) + "'";
                } else {
                    dateStr += "'" + dateList.get(i) + "'" + ",";
                }
            }
            dateStr = "(" + dateStr + ")";
            RankInfoVO longRankInfo =
                    seoKeywordsMapper.queryKeywordNumByRankRange(
                            siteCode, historyId, rankStart, rankEnd, dateStr, "2");
            if (longRankInfo != null) {
                KeywordsRankMap.put("longTail", longRankInfo.getKeywordNum());
            }
        }
        return KeywordsRankMap;
    }

    /**
     * 给列表中的关键字增加排名信息
     * 根据关键词类型
     * @param keywordList 关键词列表
     *
     */
    @Override
    public void addRankInfoByIntervalTimeByType(List<SeoKeywords> keywordList, Integer keywordType, String siteCode, String historyId, List<String> dateList) throws Exception {
        if (ListUtil.isEmpty(keywordList)) {
            return;
        }

        String datesStr = "";
        if(ListUtil.isEmpty(dateList)){
            return;
        }else{
            for(int i = 0; i < dateList.size(); i++){
                if(i == dateList.size() - 1){
                    datesStr +=  "'" + dateList.get(i) + "'";
                }else{
                    datesStr +=  "'" + dateList.get(i) + "'" + ",";
                }
            }
            datesStr = "(" + datesStr + ")";
        }
        List<Integer> keywordIds = keywordList.stream().map(SeoKeywords::getId).collect(Collectors.toList());
        List<SeoKeywordsSerp> serpList = seoKeywordsSerpService.list(new LambdaQueryWrapper<SeoKeywordsSerp>()
                .in(SeoKeywordsSerp::getKeywordsId, keywordIds)
                .in(SeoKeywordsSerp::getSeDate, dateList));
        List<SeoKeywordsSerp> noRepeatSerpList = new ArrayList<>();
        Map<String, Integer> serpMap = new HashMap<>();
        if(ListUtil.notEmpty(serpList)){
            for (SeoKeywordsSerp seoKeywordsSerp : serpList) {
                seoKeywordsSerp.setKey(seoKeywordsSerp.getKeywordsId() + "_" + seoKeywordsSerp.getSeDate());
                if(ListUtil.notEmpty(noRepeatSerpList)){
                    List<SeoKeywordsSerp> have = noRepeatSerpList.stream().filter(o -> o.getKey().equals(seoKeywordsSerp.getKey())).collect(Collectors.toList());
                    if(ListUtil.isEmpty(have)){
                        serpMap.put(seoKeywordsSerp.getKey(),seoKeywordsSerp.getRankAbsolute());
                        noRepeatSerpList.add(seoKeywordsSerp);
                    }else{
                        log.info("重复的数据：" + seoKeywordsSerp.getKey());
                    }
                }else {
                    serpMap.put(seoKeywordsSerp.getKey(),seoKeywordsSerp.getRankAbsolute());
                    noRepeatSerpList.add(seoKeywordsSerp);
                }
            }
        }

        for (SeoKeywords keyword : keywordList) {
            Map<String, String> rankInfo = new LinkedHashMap<>();
            for (String date : dateList) {

                // 从Map中获取那天的排名信息
                Integer rankAbsolute = serpMap.get(keyword.getId() + "_" + date);

                if(rankAbsolute == null){
                    rankAbsolute = 0;
                }
                rankInfo.put(date, rankAbsolute + "");
            }

            keyword.setRankInfo(rankInfo);
            keyword.setIpAddress("223.112.18.226");

            keyword.setDateList(dateList);
            if(StringUtil.isEmpty(keyword.getSearchUrl())){
                keyword.setSearchUrl("https://www.google.com/");
            }
        }
    }

    /**
     * 自适应宽度(中文支持)
     * @param sheet
     * @param size 因为for循环从0开始，size值为 列数-1
     */
    @Override
    public void setSizeColumn(Sheet sheet, int size) {
        for (int columnNum = 0; columnNum <= size; columnNum++) {
            int columnWidth = sheet.getColumnWidth(columnNum) / 256;
            for (int rowNum = 0; rowNum <= sheet.getLastRowNum(); rowNum++) {
                Row currentRow;
                //当前行未被使用过
                if (sheet.getRow(rowNum) == null) {
                    currentRow = sheet.createRow(rowNum);
                } else {
                    currentRow = sheet.getRow(rowNum);
                }

                if (currentRow.getCell(columnNum) != null) {
                    Cell currentCell = currentRow.getCell(columnNum);
                    if (currentCell.getCellType() == CellType.STRING) {
                        int length = currentCell.getStringCellValue().getBytes().length;
                        if (columnWidth < length) {
                            columnWidth = length;
                        }
                    }
                }
            }
            sheet.setColumnWidth(columnNum, columnWidth * 256);
        }
    }
}
